/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.engine.grid.integertable;

import java.util.ArrayList;
import java.util.List;

import com.vaadin.data.util.converter.Converter.ConversionException;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Component;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;

import fr.amapj.common.FormatUtils;
import fr.amapj.view.engine.grid.ErreurSaisieException;
import fr.amapj.view.engine.grid.GridIJData;
import fr.amapj.view.engine.grid.ShortCutManager;
import fr.amapj.view.engine.grid.ShortCutManager.ShortCutLine;
import fr.amapj.view.engine.notification.NotificationHelper;
import fr.amapj.view.engine.popup.corepopup.CorePopup;
import fr.amapj.view.engine.tools.BaseUiTools;
import fr.amapj.view.engine.widgets.QteTextFieldConverter;

/**
 * Popup pour la saisie des quantites en mode Table (une seule date) 
 *  
 */
@SuppressWarnings("serial")
abstract public class PopupIntegerTable extends CorePopup
{

	private Label prixTotal;

	protected IntegerTableParam param = new IntegerTableParam();

	private ShortCutManager shortCutManager;
	
	private Label labelMessageSpecifiqueBottom;
	
	private List<TextField> tfs = new ArrayList<>();
	
	private int nbRow;

	/**
	 * 
	 */
	public PopupIntegerTable()
	{

	}

	abstract public void loadParam();

	/**
	 * Retourne true si il faut fermer le popup
	 * @return
	 */
	abstract public boolean performSauvegarder();
	

	protected void createContent(VerticalLayout mainLayout)
	{
		setType(PopupType.CENTERFIT);
		loadParam();

		if (param.messageSpecifique != null)
		{
			Label messageSpeLabel = new Label(param.messageSpecifique,ContentMode.HTML);
			messageSpeLabel.addStyleName("popup-integer-table-message");
			mainLayout.addComponent(messageSpeLabel);
		}
		
		nbRow = getNbRow();
		
		//
		shortCutManager = new ShortCutManager(computeShortCutManager());
		shortCutManager.addShorcut(this.getWindow());
		
		GridLayout gd = new GridLayout(param.nbCellPerCol, nbRow);
		gd.setSpacing(true);
		mainLayout.addComponent(gd);
		
		for (int i = 0; i < param.nbCell; i++)
		{
			IntegerTableCell paramCell = param.cell[i];
			
			int row = i / param.nbCellPerCol;
			int col = i % param.nbCellPerCol;
			
			gd.addComponent(createCell(paramCell,row,col,i));
			
		}

		// Footer 0 pour avoir un espace
		HorizontalLayout footer0 = new HorizontalLayout();
		footer0.setWidth("200px");
		footer0.setHeight("20px");
		mainLayout.addComponent(footer0);

		// Footer 1 avec le prix total
		if (param.hasLignePrixTotal)
		{
			prixTotal = new Label("",ContentMode.HTML);
			displayMontantTotal();	
			mainLayout.addComponent(prixTotal);
		}
		
		// Message spécifique en bas de popup
		if (param.messageSpecifiqueBottom != null)
		{
			labelMessageSpecifiqueBottom = new Label(param.messageSpecifiqueBottom,ContentMode.HTML);
			labelMessageSpecifiqueBottom.addStyleName("popup-integer-table-message");
			mainLayout.addComponent(labelMessageSpecifiqueBottom);
		}

	}
		
	private Component createCell(IntegerTableCell paramCell, int row, int col,int cellNumber)
	{
		Panel p = new Panel();
		p.addStyleName("popup-integer-table-panel");
		p.setWidth(param.cellWidth+"px");
		p.setHeight("100%");
		
		
		VerticalLayout vl = new VerticalLayout();
		vl.setSizeFull();
		vl.setMargin(true);
		
		BaseUiTools.addStdLabel(vl, paramCell.lib1, "");
		BaseUiTools.addStdLabel(vl, paramCell.lib2, "");
		
		
		VerticalLayout vl2 = new VerticalLayout();
		vl2.setSizeFull();
		vl.addComponent(vl2);
		vl.setExpandRatio(vl2, 1f);
		
		
		HorizontalLayout hl = new HorizontalLayout();
		hl.setSpacing(true);
		hl.setWidth("100%");

		Label prixLabel = BaseUiTools.addHtmlLabel(hl, "<b>"+FormatUtils.prix(paramCell.prix)+"</b>", "");
		hl.setComponentAlignment(prixLabel, Alignment.MIDDLE_LEFT);
		
		TextField tf = createTextField(paramCell,row,col,cellNumber);
		hl.addComponent(tf);
		hl.setComponentAlignment(tf, Alignment.MIDDLE_RIGHT);
		
		
		vl.addComponent(hl);
		vl.setExpandRatio(hl, 0);
				
		p.setContent(vl);
		p.setHeight(param.cellHeight+"px");
		
		return p;
	}

	private TextField createTextField(IntegerTableCell c, int row, int col, int cellNumber)
	{
		TextField tf = new TextField();
		tf.setNullRepresentation("");
		tf.setImmediate(true);
		tfs.add(tf);
		tf.setWidth("70px");
		tf.addStyleName("cell-saisie");
		tf.setData(new GridIJData(row, col));
		shortCutManager.registerTextField(tf);
			
		if (c.isStaticText)
		{
			tf.setValue(c.staticText);
			tf.setEnabled(false);
			return tf;
		}
		
		tf.setConverter(new QteTextFieldConverter());
		tf.setConvertedValue(new Integer(c.qte));
		
		if (param.readOnly)
		{
			tf.setReadOnly(true);
			return tf;
		}
		
		tf.addValueChangeListener(e->handleTextFieldChange(tf,cellNumber));
		return tf;	
	}

	private void handleTextFieldChange(TextField tf, int cellNumber)
	{
		try
		{
			GridIJData ij = (GridIJData) tf.getData();
			int qte = readValueInCell(tf);
			param.updateQte(cellNumber, qte);
			displayMontantTotal();
		}
		catch (ErreurSaisieException e)
		{
			NotificationHelper.displayNotificationQte();
		}
	}

	private int getNbRow()
	{
		int r = param.nbCell % param.nbCellPerCol;
		if (r==0)
		{
			return param.nbCell / param.nbCellPerCol;
		}
		else
		{
			return param.nbCell / param.nbCellPerCol+1;
		}
	}


	private List<ShortCutLine> computeShortCutManager()
	{
		List<ShortCutLine> res = new ArrayList<>();
		for (int i = 0; i < nbRow; i++)
		{
			ShortCutLine line = new ShortCutLine();
			line.isVisible = true;
			line.editable = new boolean[param.nbCellPerCol];
			
			for (int j = 0; j < param.nbCellPerCol; j++)
			{
				line.editable[j] = isEditable(i,j);
			}
			
			res.add(line);
		}
		return res;
	}


	private boolean isEditable(int i, int j)
	{
		int cellNumber = i*param.nbCellPerCol+j;
		if (cellNumber>=param.nbCell)
		{
			return false;
		}
		return param.cell[cellNumber].isStaticText==false;
	}

	protected void createButtonBar()
	{
		if (param.readOnly)
		{
			addButtonBlank();
			addDefaultButton(param.libButtonSave, e->handleContinuer());
		}
		else
		{
			addButtonBlank();
			
			addButton("Annuler", e->handleAnnuler());
			
			Button saveButton = addDefaultButton(param.libButtonSave, e->handleSauvegarder());
			saveButton.addStyleName("primary");
			
		}
	}


	private void displayMontantTotal()
	{
		if (prixTotal!=null)
		{
			prixTotal.setValue("<b>"+param.libPrixTotal+" : "+FormatUtils.prix(param.getMontantTotal())+"</b>");	
		}
		
	}

	
	

	/**
	 * Retourne la valeur dans la cellule sous la forme d'un entier
	 * jette une exception si il y a une erreur
	 */
	private int readValueInCell(TextField tf) throws ErreurSaisieException
	{
		try
		{
			Integer val = (Integer) tf.getConvertedValue();
			int qte = 0;
			if (val != null)
			{
				qte = val.intValue();
			}
			return qte;
		}
		catch (ConversionException e)
		{
			throw new ErreurSaisieException();
		}
	}

	
	protected void handleAnnuler()
	{
		close();
	}
	
	/**
	 * Cette méthode est appelée quand l'utilisateur clique sur "Continuer" et que l'on est en read Only 
	 */
	protected void handleContinuer()
	{
		close();
	}
	
	/**
	 * Cette méthode est appelée quand l'utilisateur clique sur "Continuer" et que l'on N'est PAS en read Only 
	 */	
	protected void handleSauvegarder()
	{
		try
		{
			updateModele();
		}
		catch (ErreurSaisieException e)
		{
			NotificationHelper.displayNotificationQte();
			return;
		}
		
		if ((param.allowedEmpty==false) && (param.isEmpty()==true))
		{
			NotificationHelper.displayNotification("Vous devez saisir une quantité avant de continuer");
			return;
		}

		boolean ret = performSauvegarder();
		if (ret==true)
		{
			close();
		}
	}

	

	/**
	 * Lecture de la table pour mettre à jour param.cell.qte
	 */
	private void updateModele() throws ErreurSaisieException
	{
		for (int i = 0; i < param.nbCell; i++)
		{
			IntegerTableCell c = param.cell[i];
					
			if (c.isStaticText==false)
			{
				TextField tf = tfs.get(i);
				int qte = readValueInCell(tf);
				c.qte = qte;
			}
		}
	}
	
	protected void updateLabelMessageSpecifiqueBottom(String msg)
	{
		labelMessageSpecifiqueBottom.setValue(msg);
	}

	/**
	 * Cette methode doit etre appele apres la modification de la matrice de prix pour affichage correct du prix total  
	 */
	protected void updatePrixTotal()
	{
		displayMontantTotal();
	}
	
}
